@@ -15,7 +15,7 @@ __tail_doc__ = """  input_shape: it should have exactly 3 inputs channels, like
   classifier_activation: A `str` or callable. The activation function to use on the "top" layer if `num_classes > 0`.
       Set `classifier_activation=None` to return the logits of the "top" layer.
   dropout: dropout rate if top layers is included.
-  pretrained: None available.
+  pretrained: None or "imagenet". Only CMTTiny pretrained available.
   **kwargs: other parameters if available.
 
 Returns:
