@@ -66,11 +66,15 @@ def get_anchor_free_anchors(input_shape=(512, 512, 3), pyramid_levels=[3, 5], gr
 
 
 def get_yolor_anchors(input_shape=(640, 640), pyramid_levels=[3, 5], offset=0.5):
-    assert max(pyramid_levels) - min(pyramid_levels) < 3
+    # assert max(pyramid_levels) - min(pyramid_levels) < 3
 
     # width first to height first
-    # anchor_ratios = tf.convert_to_tensor([[[12.0, 16], [19, 36], [40, 28]], [[36, 75], [76, 55], [72, 146]], [[142, 110], [192, 243], [459, 401]]])
-    anchor_ratios = tf.convert_to_tensor([[[16.0, 12], [36, 19], [28, 40]], [[75, 36], [55, 76], [146, 72]], [[110, 142], [243, 192], [401, 459]]])
+    if max(pyramid_levels) - min(pyramid_levels) < 3:
+        anchor_ratios = tf.convert_to_tensor([[[16.0, 12], [36, 19], [28, 40]], [[75, 36], [55, 76], [146, 72]], [[110, 142], [243, 192], [401, 459]]])
+    else:
+        anchor_ratios = tf.convert_to_tensor(
+            [[[27.0, 19], [40, 44], [94, 38]], [[68, 96], [152, 86], [137, 180]], [[301, 140], [264, 303], [542, 238]], [[615, 436], [380, 739], [792, 925]]]
+        )
     pyramid_levels = list(range(min(pyramid_levels), max(pyramid_levels) + 1))
     feature_sizes = get_feature_sizes(input_shape, pyramid_levels)
 
@@ -93,16 +97,17 @@ def get_yolor_anchors(input_shape=(640, 640), pyramid_levels=[3, 5], offset=0.5)
     return all_anchors
 
 
-def get_pyramid_levels_by_num_anchors(input_shape, num_anchors, pyramid_levels_min=3):
+def get_pyramid_levels_by_anchors(input_shape, total_anchors, num_anchors, pyramid_levels_min=3):
     feature_sizes = get_feature_sizes(input_shape, [pyramid_levels_min, pyramid_levels_min + 10])
     feature_sizes = tf.convert_to_tensor(feature_sizes, dtype="float32")
     pyramid_levels = []
     level = pyramid_levels_min
-    while num_anchors > 0:
+    total_anchors /= num_anchors
+    while total_anchors > 0:
         pyramid_levels.append(level)
         stride_hh, stride_ww = feature_sizes[0][0] / feature_sizes[level][0], feature_sizes[0][1] / feature_sizes[level][1]
         cur_num_anchors = tf.math.ceil(input_shape[0] / stride_hh) * tf.math.ceil(input_shape[1] / stride_ww)
-        num_anchors -= int(cur_num_anchors)
+        total_anchors -= int(cur_num_anchors)
         level += 1
     return pyramid_levels
 
