@@ -183,7 +183,7 @@ def curve_fit(source, target_len=10, skip=5, use_recent=40):
     return list(source[: -len(recent_source)]) + func_curv(np.arange(start_pos, len(source) + target_len), *popt).tolist()
 
 
-def plot_and_peak_scatter(ax, source_array, peak_method, label, skip_first=0, color=None, va="bottom", fontsize=12, pred_curve=0, **kwargs):
+def plot_and_peak_scatter(ax, source_array, peak_method, label, skip_first=0, color=None, va="bottom", pred_curve=0, **kwargs):
     array = source_array[skip_first:]
     for id, ii in enumerate(array):
         if tf.math.is_nan(ii):
@@ -193,7 +193,8 @@ def plot_and_peak_scatter(ax, source_array, peak_method, label, skip_first=0, co
     pp = peak_method(array)
     vv = array[pp]
     ax.scatter(pp + skip_first, vv, color=color, marker="v")
-    ax.text(pp + skip_first, vv, "{:.4f}".format(vv), va=va, ha="right", color=color, fontsize=fontsize, rotation=0)
+    # ax.text(pp + skip_first, vv, "{:.4f}".format(vv), va=va, ha="right", color=color, fontsize=fontsize, rotation=0)
+    ax.text(pp + skip_first, vv, "{:.4f}".format(vv), va=va, ha="right", color=color, rotation=0)
 
     if pred_curve > 0:
         kwargs.pop("linestyle", None)
@@ -204,14 +205,22 @@ def plot_and_peak_scatter(ax, source_array, peak_method, label, skip_first=0, co
 def plot_hists(hists, names=None, base_size=6, addition_plots=["lr"], text_va=["bottom"], skip_first=0, pred_curve=0):
     import os
     import json
-    import matplotlib.pyplot as plt
     import numpy as np
+    import matplotlib as mpl
+    import matplotlib.pyplot as plt
+
+    mpl.rcParams["lines.linewidth"] = base_size / 8 * 1.5  # 8 -> 1.5
+    mpl.rcParams["font.size"] = base_size * 2 - 4  # 8 -> 12, 6 -> 8
+    mpl.rcParams["legend.fontsize"] = base_size * 2 - 4
+    mpl.rcParams["xtick.labelsize"] = base_size * 2 - 4
+    mpl.rcParams["ytick.labelsize"] = base_size * 2 - 4
+    mpl.rcParams["xtick.major.pad"] = base_size / 2 - 2  # 8 -> 4, 6 -> 3
+    mpl.rcParams["ytick.major.pad"] = base_size / 2 - 2
 
     num_axes = (2 + len(addition_plots)) if addition_plots is not None and len(addition_plots) != 0 else 2
     fig, axes = plt.subplots(1, num_axes, figsize=(num_axes * base_size, base_size))
     hists = [hists] if isinstance(hists, (str, dict)) else hists
     names = names if isinstance(names, (list, tuple)) else [names]
-    fontsize = base_size * 2 - 2
     for id, hist in enumerate(hists):
         name = names[min(id, len(names) - 1)] if names != None else None
         cur_va = text_va[id % len(text_va)]
@@ -221,24 +230,39 @@ def plot_hists(hists, names=None, base_size=6, addition_plots=["lr"], text_va=["
                 hist = json.load(ff)
         name = name if name != None else str(id)
 
+        acc_key = "acc"
+        if acc_key not in hist:
+            all_acc_key = [ii for ii in hist.keys() if "acc" in ii and "val" not in ii]
+            acc_key = "acc" if len(all_acc_key) == 0 else all_acc_key[0]
+        val_acc_key = "val_acc"
+        if val_acc_key not in hist:
+            all_val_acc_key = [ii for ii in hist.keys() if "acc" in ii and "val" in ii]
+            val_acc_key = "val_acc" if len(all_val_acc_key) == 0 else all_val_acc_key[0]
+
         cur_pred_curve = pred_curve[min(id, len(pred_curve) - 1)] if isinstance(pred_curve, (list, tuple)) else pred_curve
-        plot_and_peak_scatter(axes[0], hist["loss"], np.argmin, name + " loss", skip_first, None, cur_va, fontsize, pred_curve=cur_pred_curve)
+        plot_and_peak_scatter(axes[0], hist["loss"], np.argmin, name + " loss", skip_first, None, cur_va, pred_curve=cur_pred_curve)
         color = axes[0].lines[-1].get_color()
         val_loss = hist.get("val_loss", [])
         if len(val_loss) > 0 and "val_loss" not in addition_plots:
-            plot_and_peak_scatter(axes[0], val_loss, np.argmin, name + " val_loss", skip_first, color, cur_va, fontsize, cur_pred_curve, linestyle="--")
-        acc = hist.get("acc", hist.get("accuracy", []))
+            plot_and_peak_scatter(axes[0], val_loss, np.argmin, name + " val_loss", skip_first, color, cur_va, cur_pred_curve, linestyle="--")
+
+        acc = hist.get(acc_key, [])
         if len(acc) > 0:  # For timm log
-            plot_and_peak_scatter(axes[1], acc, np.argmax, name + " accuracy", skip_first, color, cur_va, fontsize, cur_pred_curve)
-        val_acc = hist.get("val_acc", hist.get("val_accuracy", []))
-        plot_and_peak_scatter(axes[1], val_acc, np.argmax, name + " val_accuracy", skip_first, color, cur_va, fontsize, cur_pred_curve, linestyle="--")
+            plot_and_peak_scatter(axes[1], acc, np.argmax, name + " accuracy", skip_first, color, cur_va, cur_pred_curve)
+
+        val_acc = hist.get(val_acc_key, [])
+        if len(val_acc) > 0:  # For timm log
+            plot_and_peak_scatter(axes[1], val_acc, np.argmax, name + " val_accuracy", skip_first, color, cur_va, cur_pred_curve, linestyle="--")
+
         if addition_plots is not None and len(addition_plots) != 0:
             for id, ii in enumerate(addition_plots):
                 if len(hist.get(ii, [])) > 0:
                     peak_method = np.argmin if "loss" in ii else np.argmax
-                    plot_and_peak_scatter(axes[2 + id], hist[ii], peak_method, name + " " + ii, skip_first, color, cur_va, fontsize, cur_pred_curve)
+                    plot_and_peak_scatter(axes[2 + id], hist[ii], peak_method, name + " " + ii, skip_first, color, cur_va, cur_pred_curve)
     for ax in axes:
         ax.legend()
         ax.grid(True)
     fig.tight_layout()
+    fig.tight_layout()  # again
+    plt.show()
     return fig
